"""Configuration for the tests."""

import pytest
from syrupy import SnapshotAssertion

from .syrupy import CustomSnapshotExtension


@pytest.fixture(name="snapshot")
def snapshot_assertion(snapshot: SnapshotAssertion) -> SnapshotAssertion:
    """Return snapshot assertion fixture with the custom extension."""
    return snapshot.use_extension(CustomSnapshotExtension)



"""WebRTC models tests."""

from __future__ import annotations

from typing import TYPE_CHECKING

import orjson
import pytest

from tests import load_fixture
from webrtc_models import (
    RTCConfiguration,
    RTCIceCandidate,
    RTCIceCandidateInit,
    RTCIceServer,
)

if TYPE_CHECKING:
    from mashumaro.mixins.orjson import DataClassORJSONMixin
    from syrupy import SnapshotAssertion


@pytest.mark.parametrize(
    ("clazz", "filename"),
    [
        # RTCIceServer
        (RTCIceServer, "RTCIceServer_only_urls_string.json"),
        (RTCIceServer, "RTCIceServer_only_urls_list.json"),
        (RTCIceServer, "RTCIceServer_urls_string.json"),
        (RTCIceServer, "RTCIceServer_urls_list.json"),
        # RTCConfiguration
        (RTCConfiguration, "RTCConfiguration_empty.json"),
        (RTCConfiguration, "RTCConfiguration_one_iceServer.json"),
        (RTCConfiguration, "RTCConfiguration_multiple_iceServers.json"),
        # RTCIceCandidateInit
        (RTCIceCandidateInit, "RTCIceCandidateInit_end.json"),
        (RTCIceCandidateInit, "RTCIceCandidateInit_candidate.json"),
    ],
)
def test_decoding_and_encoding(
    snapshot: SnapshotAssertion,
    clazz: type[DataClassORJSONMixin],
    filename: str,
) -> None:
    """Test decoding/encoding."""
    file_content = load_fixture(filename)
    instance = clazz.from_json(file_content)
    assert instance == snapshot(name="dataclass")

    file_content_dict = orjson.loads(file_content)
    instance_dict = instance.to_dict()

    # Verify json
    assert instance.to_json() == orjson.dumps(file_content_dict).decode()

    # Verify dict
    assert instance_dict == file_content_dict
    assert instance == clazz.from_dict(instance_dict)


@pytest.mark.parametrize(
    ("clazz", "filename"),
    [
        # RTCIceCandidate
        (RTCIceCandidate, "RTCIceCandidate_end.json"),
        (RTCIceCandidate, "RTCIceCandidate_candidate.json"),
    ],
)
def test_decoding_and_encoding_deprecated(
    snapshot: SnapshotAssertion,
    clazz: type[DataClassORJSONMixin],
    filename: str,
) -> None:
    """Test decoding/encoding."""
    file_content = load_fixture(filename)
    with pytest.deprecated_call():
        instance = clazz.from_json(file_content)
    assert instance == snapshot(name="dataclass")

    file_content_dict = orjson.loads(file_content)
    instance_dict = instance.to_dict()

    # Verify json
    assert instance.to_json() == orjson.dumps(file_content_dict).decode()

    # Verify dict
    assert instance_dict == file_content_dict
    with pytest.deprecated_call():
        assert instance == clazz.from_dict(instance_dict)


def test_no_mid_and_mlineindex() -> None:
    """Test spd_mid and sdp_multilineindex raises TypeError."""
    file_content = load_fixture("RTCIceCandidate_candidate.json")
    cand = RTCIceCandidateInit.from_json(file_content)
    assert cand.sdp_m_line_index == 0
    assert cand.sdp_mid is None


def test_invalid_mlineindex() -> None:
    """Test spd_mid and sdp_multilineindex raises TypeError."""
    file_content = load_fixture("RTCIceCandidateInit_invalid.json")
    msg = "sdpMLineIndex must be greater than or equal to 0"
    with pytest.raises(ValueError, match=msg):
        RTCIceCandidateInit.from_json(file_content)



"""WebRTC models."""

from dataclasses import dataclass, field
from warnings import warn

from mashumaro import field_options
from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin

__all__ = [
    "RTCConfiguration",
    "RTCIceCandidate",
    "RTCIceServer",
]


class _RTCBaseModel(DataClassORJSONMixin):
    """Base class for RTC models."""

    class Config(BaseConfig):
        """Mashumaro config."""

        # Serialize to spec conform names and omit default values
        omit_default = True
        serialize_by_alias = True


@dataclass
class RTCIceServer(_RTCBaseModel):
    """RTC Ice Server.

    See https://www.w3.org/TR/webrtc/#rtciceserver-dictionary
    """

    urls: str | list[str]
    username: str | None = None
    credential: str | None = None


@dataclass
class RTCConfiguration(_RTCBaseModel):
    """RTC Configuration.

    See https://www.w3.org/TR/webrtc/#rtcconfiguration-dictionary
    """

    ice_servers: list[RTCIceServer] = field(
        metadata=field_options(alias="iceServers"), default_factory=list
    )


@dataclass(frozen=True)
class RTCIceCandidate(_RTCBaseModel):
    """RTC Ice Candidate.

    See https://www.w3.org/TR/webrtc/#rtcicecandidate-interface
    """

    candidate: str

    def __post_init__(self) -> None:
        """Initialize class."""
        msg = "Using RTCIceCandidate is deprecated. Use RTCIceCandidateInit instead"
        warn(msg, DeprecationWarning, stacklevel=2)


@dataclass(frozen=True)
class RTCIceCandidateInit(RTCIceCandidate):
    """RTC Ice Candidate Init.

    If neither sdp_mid nor sdp_m_line_index are provided and candidate is not an empty
    string, sdp_m_line_index is set to 0.
    See https://www.w3.org/TR/webrtc/#dom-rtcicecandidateinit
    """

    candidate: str
    sdp_mid: str | None = field(
        metadata=field_options(alias="sdpMid"), default=None, kw_only=True
    )
    sdp_m_line_index: int | None = field(
        metadata=field_options(alias="sdpMLineIndex"), default=None, kw_only=True
    )
    user_fragment: str | None = field(
        metadata=field_options(alias="userFragment"), default=None, kw_only=True
    )

    def __post_init__(self) -> None:
        """Initialize class."""
        if not self.candidate:
            # An empty string represents an end-of-candidates indication
            # or a peer reflexive remote candidate
            return

        if self.sdp_mid is None and self.sdp_m_line_index is None:
            object.__setattr__(self, "sdp_m_line_index", 0)
        elif (sdp := self.sdp_m_line_index) is not None and sdp < 0:
            msg = "sdpMLineIndex must be greater than or equal to 0"
            raise ValueError(msg)
